use std::env;

use actix_web::{web, HttpResponse, Responder};
use lettre::{
    message::{header::ContentType, MultiPart, SinglePart},
    Message, SmtpTransport, Transport,
};
use serde::Deserialize;
use sqlx::PgPool;
use zxcvbn::{zxcvbn, Score};

use crate::{
    auth::{self},
    models::{PasswordReset, User},
    utils::password_help,
};

#[derive(Deserialize, Debug)]
struct ResetPasswordForm {
    email: Option<String>,
    token: Option<String>,
    password: Option<String>,
    passwordretyped: Option<String>,
    dry: Option<bool>,
}

#[actix_web::post("/reset-password")]
async fn post(
    form: web::Form<ResetPasswordForm>,
    pool: web::Data<PgPool>,
    mailer: web::Data<SmtpTransport>,
) -> impl Responder {
    if form.email.is_some()
        && form.token.is_none()
        && form.password.is_none()
        && form.passwordretyped.is_none()
    {
        if let Ok(user) = User::read_for_login(pool.get_ref(), form.email.as_ref().unwrap()).await {
            let reset = PasswordReset::insert_new_for_user(pool.get_ref(), user.id)
                .await
                .unwrap();

            let hostname = env::var("HOSTNAME").unwrap();
            let reset_url = format!("https://{}/reset-password?token={}", hostname, reset.token);

            let message = Message::builder()
                .from("noreply <noreply@brasiwa-leipzig.de>".parse().unwrap())
                .reply_to("noreply <noreply@brasiwa-leipzig.de>".parse().unwrap())
                .to(format!("{} <{}>", user.name, user.email).parse().unwrap())
                .subject("Brass: Zurücksetzen des Passworts angefordert")
                .multipart(
                    MultiPart::alternative()
                        .singlepart(
                            SinglePart::builder()
                                .header(ContentType::TEXT_PLAIN)
                                .body(format!(r##"Hallo {},

du hast angefordert, dein Passwort zurückzusetzen. Kopiere dafür folgenden Link in deinen Browser:

{}

Bitte beachte, dass der Link nur 24 Stunden gültig ist. Solltest du nichts angefordert haben, dann musst du nichts weiter tun.

Viele Grüße"##, user.name, reset_url))
                )
                .singlepart(
                    SinglePart::builder()
                        .header(ContentType::TEXT_HTML)
                        .body(format!(r##"<p>Hallo {},</p>

<p>du hast angefordert, dein Passwort zurückzusetzen. Klicke dafür <a href="{}" target="_blank">hier</a> oder kopiere folgenden Link in deinen Browser:</p>

<p>{}</p>

<p>Bitte beachte, dass der Link <b>nur 24 Stunden gültig</b> ist. Solltest du nichts angefordert haben, dann musst du nichts weiter tun.</p>

<p>Viele Grüße</p>"##, user.name, reset_url, reset_url))
                ))
                .unwrap();

            mailer.send(&message).unwrap();
        }

        return HttpResponse::Ok().body("E-Mail versandt!");
    } else if form.email.is_none()
        && form.token.is_some()
        && form.password.is_some()
        && form.passwordretyped.is_some()
    {
        let password = form.password.as_ref().unwrap();
        let is_dry = form.dry.is_some_and(|b| b);

        let token =
            PasswordReset::does_token_exist(pool.get_ref(), form.token.as_ref().unwrap()).await;

        if token.is_err() {
            return HttpResponse::BadRequest().body("Token existiert nicht bzw. ist abgelaufen!");
        }

        if password.chars().count() > 256 {
            if is_dry {
                return HttpResponse::BadRequest().body("<div id=\"password-strength\" class=\"mb-3 help content is-danger\">Password darf nicht länger als 256 Zeichen sein.</div>");
            } else {
                return HttpResponse::NoContent().finish();
            }
        }

        let user = User::read_by_id(pool.get_ref(), token.as_ref().unwrap().userid)
            .await
            .unwrap();
        let mut split_names: Vec<&str> = user.name.as_str().split_whitespace().collect();
        let mut user_inputs = vec![user.email.as_str()];
        user_inputs.append(&mut split_names);
        let entropy = zxcvbn(password, &user_inputs);

        if entropy.score() < Score::Three {
            if is_dry {
                let message = password_help::generate_for_entropy(&entropy);

                return HttpResponse::BadRequest().body(message);
            } else {
                return HttpResponse::NoContent().finish();
            }
        }

        if is_dry {
            if entropy.score() == Score::Three {
                return HttpResponse::Ok()
                    .body("<div id=\"password-strength\" class=\"mb-3 help content is-success\">Sicheres Passwort.</div>");
            } else {
                return HttpResponse::Ok()
                    .body("<div id=\"password-strength\" class=\"mb-3 help content is-success\">Sehr sicheres Passwort.</div>");
            }
        }

        if password != form.passwordretyped.as_ref().unwrap() {
            return HttpResponse::BadRequest().body("Passwörter stimmen nicht überein!");
        }

        let (hash, salt) =
            auth::utils::generate_salt_and_hash_plain_password(form.password.as_ref().unwrap())
                .unwrap();

        User::update(
            pool.get_ref(),
            token.as_ref().unwrap().userid,
            None,
            None,
            Some(&hash),
            Some(&salt),
            None,
            None,
            None,
            None,
            None,
        )
        .await
        .unwrap();

        PasswordReset::delete(pool.get_ref(), &token.unwrap().token)
            .await
            .unwrap();

        return HttpResponse::Ok().body(r#"<div class="block">Passwort wurde geändert.</div><a class="block button is-primary" hx-boost="true" href="/login">Zum Login</a>"#);
    } else {
        return HttpResponse::BadRequest().finish();
    }
}
